//-------------------------------------------------------------
// Acclight: experimental environment for 
//           multipass-rendering, lighting, shading and shadows
//-------------------------------------------------------------
//
// Daniel Bielser - january  1999
// Reto Ltolf    - december 1999
//-------------------------------------------------------------


#include "acclight.h"


// full screen anti-aliasing using camera jittering
//#define AL_ANTI_ALIASING



//light properties :
//  array to hold the different light_positions
float light_position[AL_NBR_OF_LIGHTS][4];
//  random values for camera jittering
GLdouble j8[AL_NBR_JITTER_SAMPLE][2] = {{0.375, 0.4375},{0.625, 0.0625},{0.875, 0.1875},{0.125 ,0.0625},
					{0.375, 0.6875},{0.875, 0.4375},{0.625, 0.5625},{0.375, 0.9375},
					{0.625, 0.3125},{0.125, 0.5625},{0.125, 0.8125},{0.375, 0.1875},
					{0.875, 0.9375},{0.875, 0.6875},{0.125, 0.3125},{0.625, 0.8125}};


//material properties :
//  reflexion properties of the table
GLfloat table_specular[] = { 0.5, 0.5, 0.5, 1.0 };					   
GLfloat table_diffuse[]  = { 1.0, 1.0, 1.0, 1.0 };
GLfloat table_ambient[]  = { (float)0.2, (float)0.2, (float)0.2, 1.0 };
GLfloat table_shininess  = 100.0;
GLfloat table_emission[] = {0.0, 0.0, 0.0, 0.0};

//  reflection properties of the teapot
GLfloat teapot_specular[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat teapot_diffuse[] = { (float)0.2, (float)0.2, (float)0.2, 1.0 };
GLfloat teapot_ambient[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat teapot_shininess = 50.0;
GLfloat teapot_emission[] = {0.0, 0.0, 0.0, 0.0};

//  reflection properties of the shadow
GLfloat shadow_specular[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat shadow_diffuse[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat shadow_ambient[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat shadow_shininess = 50.0;
GLfloat shadow_emission[] = {0.0, 0.0, 0.0, 0.0};

//  reflection properites of the walls
GLfloat wall_specular[] = { 0.5, 0.5, 0.5, 1.0 };
GLfloat wall_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat wall_ambient[] = { 0.5, 0.5, 0.5, 1.0 };
GLfloat wall_shininess = 50.0;
GLfloat wall_emission[] = {0.0, 0.0, 0.0, 0.0};

//  emission properties of the drawed lights
GLfloat light_specular[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat light_diffuse[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat light_ambient[] = { 0.0, 0.0, 0.0, 1.0 };
GLfloat light_shininess = 50.0;
GLfloat light_emission[] = {1.0, 1.0, 0.5, 0.0};

//  texturing
GLint wall_texture_width, wall_texture_height;
GLint wood_texture_width, wood_texture_height;
GLint carpet_texture_width, carpet_texture_height;
GLubyte wall_texture[AL_TEXTURE_X_SIZE][AL_TEXTURE_Y_SIZE][3];
GLubyte wood_texture[AL_TEXTURE_X_SIZE][AL_TEXTURE_Y_SIZE][3];
GLubyte carpet_texture[AL_TEXTURE_X_SIZE][AL_TEXTURE_Y_SIZE][3];
GLuint  texture_ids[3];
GLuint wall_texture_id   = 0;
GLuint wood_texture_id   = 1;
GLuint carpet_texture_id = 2;



//functions :
int read_texmap(char *acFile, int *width, int *height,
		GLubyte TexMap[AL_TEXTURE_X_SIZE][AL_TEXTURE_Y_SIZE][3])
{
  FILE *file;
  int i, j, xsize, ysize, maxval, iTexOk;
  unsigned char ch, buf[200];
  
  file = fopen(acFile, "rb");
  if(file==NULL){
    printf("ERROR: dwsa_read_texmap: bad fileopen\n");
    glDisable(GL_TEXTURE_2D);
    iTexOk = 0;
    return AL_FILE_ERROR;
  }

  // read header
  fread(buf, 2, 1, file); //magic number..
  if((buf[0] != 'P')||(buf[1] != '6')) {
    printf("ERROR: dwsa_read_texmap: bad file magicnum\n");
    glDisable(GL_TEXTURE_2D);
    iTexOk = 0;
    return AL_FILE_ERROR;
  }

  // skip whitespace
  fread(&buf[0], 1, 1, file);
  do {
    ch = buf[0];
    while( ch != '\n' ) fread(&ch, 1, 1, file);
    fread(&buf[0], 1, 1, file);
  } while( (buf[0] < '0') || (buf[0] > '9') );

  //get width
  fread(&ch, 1, 1, file);
  for(i = 1; (ch >= '0') && (ch <= '9'); i++){
    buf[i] = ch;
    fread(&ch, 1, 1, file);
  }
  buf[i] = '\0';
  xsize = atoi((char *)buf);

  // the space has been read, can continue straight
  fread(&ch, 1, 1, file);
  for(i = 0; (ch >= '0') && (ch <= '9'); i++){
    buf[i] = ch;
    fread(&ch, 1, 1, file);
  }
  buf[i] = '\0';
  ysize = atoi((char *)buf);
  
  //the newline has been read, can continue straight
  fread(&ch, 1, 1, file);
  for(i = 0; (ch >= '0') && (ch <= '9'); i++){
    buf[i] = ch;
    fread(&ch, 1, 1, file);
  }
  buf[i] = '\0';
  maxval = atoi((char *)buf);
  
  if((xsize != ysize) || ( (xsize != 512) && (xsize != 256) && 
			   (xsize != 128) && (xsize != 64) && (xsize != 32) && (xsize != 16))){
    printf("ERROR: dwsa_read_texmap: bad file dims\n");
    glDisable(GL_TEXTURE_2D);
    iTexOk = 0;
    return AL_FILE_ERROR;
  }
  
  // set up OpenGL to use texture mapping
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

  // the newline is read, continue..
  for(i = 0; i < xsize; i++){
    for(j = 0; j < ysize; j++){
      fread(&ch, 1, 1, file);
      TexMap[xsize-1-i][j][0] = ch;
      fread(&ch, 1, 1, file);
      TexMap[xsize-1-i][j][1] = ch;
      fread(&ch, 1, 1, file);
      TexMap[xsize-1-i][j][2] = ch;
    }
  }

  fclose(file);
  iTexOk = 1;
  *width = xsize;
  *height = ysize;
  return 0;
}


void accFrustum(GLdouble left, GLdouble right, GLdouble bottom, GLdouble top,
		GLdouble near_, GLdouble far_, GLdouble pixdx, GLdouble pixdy,
		GLdouble eyedx, GLdouble eyedy, GLdouble focus)
{
  GLdouble  xwsize, ywsize;
  GLdouble  dx, dy;
  GLint     viewport[4];

  glGetIntegerv (GL_VIEWPORT, viewport);

  xwsize = right - left;
  ywsize = top - bottom;
  dx = -(pixdx*xwsize/(GLdouble)viewport[2] + eyedx*near_/focus);
  dy = -(pixdy*ywsize/(GLdouble)viewport[3] + eyedy*near_/focus);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum (left + dx, right + dx, bottom + dy, top + dy, near_, far_); 
  gluLookAt(AL_CAM_POS_X, AL_CAM_POS_Y, AL_CAM_POS_Z, AL_CENTER_X+0.8, AL_CENTER_Y, AL_CENTER_Z, 0.0, 1.0, 0.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef (-eyedx, -eyedy, 0.0);
}


void accPerspective(GLdouble fovy, GLdouble aspect, GLdouble near_, GLdouble far_,
		    GLdouble pixdx, GLdouble pixdy, GLdouble eyedx, GLdouble eyedy,
		    GLdouble focus)
{
  GLdouble fov2,left,right,bottom,top;

  fov2 = ((fovy*AL_PI) / 180.0) / 2.0;
  top = near_ / (cos(fov2) / sin(fov2));
  bottom = -top;
  right = top * aspect;
  left = -right;

  accFrustum (left, right, bottom, top, near_, far_, pixdx, pixdy, eyedx, eyedy, focus);
}


void setLighting(void)
{
  // global ambient light
  GLfloat global_ambient[4];  

  // attentuation
  // factor = 1/(constant_att + linear_att*dist + quadratic_att*sqr(dist))
  GLfloat constant_attenuation; 
  GLfloat linear_attenuation;
  GLfloat quadratic_attenuation;
  
  // parameters for spot-light characteristic
  GLfloat spot_direction[3];
  GLfloat spot_exponent;
  GLfloat spot_cutoff;
  
  //define the above variables...
 


  // set the global ambient light
  // using 'glLightModelfv'
  

  // set the attenuation parameters for light GL_LIGHT0
  // using 'glLightf'
  

  
  // set parameters for spotlighting
  // using 'glLightfv' and 'glLightf'
  

  
  // set the lighting model such that specular reflexions
  // are dependent on camera position
  // (using 'glLightModeli' with parameter GL_LIGHT_MODEL_LOCAL_VIEWER
  //  and ... )
  
  
  // enable lighting in general and enable GL_LIGHT0
  // using 'glEnable'
  

}


void setMaterial(GLfloat *ambient, GLfloat *diffuse, GLfloat *specular, 
		 GLfloat shininess, GLfloat *emission)
{
  // set the ambient reflection coefficient
  // using 'glMaterialfv'
  

  // set the diffuse reflection coefficient
  // using 'glMaterialfv'
  

  // set the specular reflection coefficient
  // using 'glMaterialfv'
  

  // set the shininess (using 'glMaterialf')
  
  
  // set the self emission (using 'glMaterialfv')
  
}


void buildTexturedRectangle(float rectangle_width, float rectangle_height,
			    float rectangle_depth, float sub_div)
{
  int i,j;
  float half_width, half_height, half_depth;

  half_width = rectangle_width / 2;
  half_height = rectangle_height / 2;
  half_depth = rectangle_depth / 2;
  
  // Y=0-face
  if(rectangle_height == 0.0) {
    for(i = 0; i < sub_div; i++) {
      for(j = 0; j < sub_div; j++) {
	glBegin(GL_QUADS);
	glNormal3f(0.0, 1.0, 0.0);              
	glTexCoord2f(i/sub_div*AL_WALL_TEX_SCAL, j/sub_div*AL_WALL_TEX_SCAL); 
	glVertex3f(-half_width + rectangle_width*i/sub_div, 0.0, -half_depth + rectangle_depth*j/sub_div); 
	glTexCoord2f(i/sub_div*AL_WALL_TEX_SCAL, (j+1)/sub_div*AL_WALL_TEX_SCAL);
	glVertex3f(-half_width + rectangle_width*i/sub_div, 0.0, -half_depth + rectangle_depth*(j+1)/sub_div);
	glTexCoord2f((i+1)/sub_div*AL_WALL_TEX_SCAL, (j+1)/sub_div*AL_WALL_TEX_SCAL);
	glVertex3f(-half_width + rectangle_width*(i+1)/sub_div, 0.0, -half_depth + rectangle_depth*(j+1)/sub_div);
	glTexCoord2f((i+1)/sub_div*AL_WALL_TEX_SCAL, (j)/sub_div*AL_WALL_TEX_SCAL);
	glVertex3f(-half_width + rectangle_width*(i+1)/sub_div, 0.0, -half_depth + rectangle_depth*j/sub_div);
	glEnd();
      }
    }
  }
  // X=0-face
  if(rectangle_width == 0.0) {
    for(i = 0; i < sub_div; i++) {
      for(j = 0; j < sub_div; j++) {
	glBegin(GL_QUADS);
	glNormal3f(-1.0, 0.0, 0.0);
	glTexCoord2f((i+1)/sub_div*AL_WALL_TEX_SCAL, (j)/sub_div*AL_WALL_TEX_SCAL);
	glVertex3f(0.0, -half_height + rectangle_height*(i+1)/sub_div, -half_depth + rectangle_depth*j/sub_div);
	glTexCoord2f((i+1)/sub_div*AL_WALL_TEX_SCAL, (j+1)/sub_div*AL_WALL_TEX_SCAL);
	glVertex3f(0.0, -half_height + rectangle_height*(i+1)/sub_div, -half_depth + rectangle_depth*(j+1)/sub_div);
	glTexCoord2f(i/sub_div*AL_WALL_TEX_SCAL, (j+1)/sub_div*AL_WALL_TEX_SCAL);
	glVertex3f(0.0, -half_height + rectangle_height*i/sub_div, -half_depth + rectangle_depth*(j+1)/sub_div);
	glTexCoord2f(i/sub_div*AL_WALL_TEX_SCAL, j/sub_div*AL_WALL_TEX_SCAL); 
	glVertex3f(0.0, -half_height + rectangle_height*i/sub_div, -half_depth + rectangle_depth*j/sub_div);
	glEnd();
      }
    }
  }
  // Z=0-face
  if(rectangle_depth == 0.0) {
    for(i = 0; i < sub_div; i++) {
      for(j = 0; j < sub_div; j++) {
	glBegin(GL_QUADS);//glBegin(GL_POLYGON);
	glNormal3f(0.0, 0.0, -1.0);     
	glTexCoord2f(i/sub_div*AL_WALL_TEX_SCAL, j/sub_div*AL_WALL_TEX_SCAL); 
	glVertex3f(-half_width + rectangle_width*i/sub_div, -half_height + rectangle_height*j/sub_div, 0.0);
	glTexCoord2f(i/sub_div*AL_WALL_TEX_SCAL, (j+1)/sub_div*AL_WALL_TEX_SCAL);
	glVertex3f(-half_width + rectangle_width*i/sub_div, -half_height + rectangle_height*(j+1)/sub_div, 0.0);
	glTexCoord2f((i+1)/sub_div*AL_WALL_TEX_SCAL, (j+1)/sub_div*AL_WALL_TEX_SCAL);
	glVertex3f(-half_width + rectangle_width*(i+1)/sub_div, -half_height + rectangle_height*(j+1)/sub_div, 0.0);
	glTexCoord2f((i+1)/sub_div*AL_WALL_TEX_SCAL, (j)/sub_div*AL_WALL_TEX_SCAL);
	glVertex3f(-half_width + rectangle_width*(i+1)/sub_div, -half_height + rectangle_height*j/sub_div, 0.0);
	glEnd();
      }
    }	  
  }
}



void buildTexturedCube(float cube_width, float cube_height, float cube_depth)
{
  float half_width, half_height, half_depth;

  half_width = cube_width / 2;
  half_height = cube_height / 2;
  half_depth = cube_depth / 2;
  
  // top
  glBegin(GL_POLYGON);
  glNormal3f(0.0, 1.0, 0.0);
  glTexCoord2f(0.0, 0.0); glVertex3f(-half_width, half_height, -half_depth);
  glTexCoord2f(0.0, cube_depth*AL_TEX_SCAL); glVertex3f(-half_width, half_height, half_depth);
  glTexCoord2f(cube_width*AL_TEX_SCAL, 0.0); glVertex3f(half_width, half_height, half_depth);
  glTexCoord2f(cube_width*AL_TEX_SCAL, cube_depth*AL_TEX_SCAL); glVertex3f(half_width, half_height, -half_depth);
  glEnd();
  // bottom
  glBegin(GL_POLYGON);
  glNormal3f(0.0, -1.0, 0.0);
  glTexCoord2f(0.0, 0.0); glVertex3f(-half_width, -half_height, -half_depth);
  glTexCoord2f(0.0, cube_depth*AL_TEX_SCAL); glVertex3f(-half_width, -half_height, half_depth);
  glTexCoord2f(cube_width*AL_TEX_SCAL, 0.0); glVertex3f(half_width, -half_height, half_depth);
  glTexCoord2f(cube_width*AL_TEX_SCAL, cube_depth*AL_TEX_SCAL); glVertex3f(half_width, -half_height, -half_depth);
  glEnd();
  // left
  glBegin(GL_POLYGON);
  glNormal3f(-1.0, 0.0, 0.0);
  glTexCoord2f(0.0, 0.0); glVertex3f(-half_width, half_height, -half_depth);
  glTexCoord2f(0.0, cube_depth*AL_TEX_SCAL); glVertex3f(-half_width, half_height, half_depth);
  glTexCoord2f(cube_height*AL_TEX_SCAL, 0.0); glVertex3f(-half_width, -half_height, half_depth);
  glTexCoord2f(cube_height*AL_TEX_SCAL, cube_depth*AL_TEX_SCAL); glVertex3f(-half_width, -half_height, -half_depth);
  glEnd();
  // right
  glBegin(GL_POLYGON);
  glNormal3f(1.0, 0.0, 0.0);
  glTexCoord2f(0.0, 0.0); glVertex3f(half_width, half_height, -half_depth);
  glTexCoord2f(0.0, cube_depth*AL_TEX_SCAL); glVertex3f(half_width, half_height, half_depth);
  glTexCoord2f(cube_height*AL_TEX_SCAL, 0.0); glVertex3f(half_width, -half_height, half_depth);
  glTexCoord2f(cube_height*AL_TEX_SCAL, cube_depth*AL_TEX_SCAL); glVertex3f(half_width, -half_height, -half_depth);
  glEnd();
  // front
  glBegin(GL_POLYGON);
  glNormal3f(0.0, 0.0, 1.0);
  glTexCoord2f(0.0, 0.0); glVertex3f(-half_width, -half_height, half_depth);
  glTexCoord2f(0.0, cube_height*AL_TEX_SCAL); glVertex3f(-half_width, half_height, half_depth);
  glTexCoord2f(cube_width*AL_TEX_SCAL, 0.0); glVertex3f(half_width, half_height, half_depth);
  glTexCoord2f(cube_width*AL_TEX_SCAL, cube_height*AL_TEX_SCAL); glVertex3f(half_width, -half_height, half_depth);
  glEnd();
  // back
  glBegin(GL_POLYGON);
  glNormal3f(0.0, 0.0, -1.0);
  glTexCoord2f(0.0, 0.0); glVertex3f(-half_width, -half_height, -half_depth);
  glTexCoord2f(0.0, cube_height*AL_TEX_SCAL); glVertex3f(-half_width, half_height, -half_depth);
  glTexCoord2f(cube_width*AL_TEX_SCAL, 0.0); glVertex3f(half_width, half_height, -half_depth);
  glTexCoord2f(cube_width*AL_TEX_SCAL, cube_height*AL_TEX_SCAL); glVertex3f(half_width, -half_height, -half_depth);
  glEnd();
}  
  

void buildTable(void) 
{
  glNewList(AL_TABLE_LIST,GL_COMPILE);
  
  // table top
  glPushMatrix ();
  buildTexturedCube((float)2.0, (float)0.1, (float)1.0);
  glPopMatrix ();

  // table-legs
  glPushMatrix ();
  glTranslatef ((float)0.95, (float)-0.55,  (float)0.45);
  buildTexturedCube((float)0.1, (float)1.0, (float)0.1);
  glPopMatrix ();

  glPushMatrix ();
  glTranslatef ((float)0.95, (float)-0.55, (float)-0.45);
  buildTexturedCube((float)0.1, (float)1.0, (float)0.1);
  glPopMatrix ();

  glPushMatrix ();
  glTranslatef ((float)-0.95, (float)-0.55, (float)0.45);
  buildTexturedCube((float)0.1, (float)1.0, (float)0.1);
  glPopMatrix ();

  glPushMatrix ();
  glTranslatef ((float)-0.95, (float)-0.55, (float)-0.45);
  buildTexturedCube((float)0.1, (float)1.0, (float)0.1);
  glPopMatrix ();
   
  glEndList();
}


void buildChair(void) 
{
  glNewList(AL_CHAIR_LIST,GL_COMPILE);
   
  glPushMatrix();
  glTranslatef((float)0.0, (float)-0.05, (float)0.0);
   
  // chair top
  glPushMatrix ();
  glTranslatef ((float)0.0, (float)-0.255,  (float)-1.0);
  buildTexturedCube((float)0.5, (float)0.05, (float)0.5);
  glPopMatrix ();

  // chair legs
  glPushMatrix ();
  glTranslatef ((float)0.25, (float)-0.625,  (float)-0.75);
  buildTexturedCube((float)0.05, (float)0.75, (float)0.05);
  glPopMatrix ();

  glPushMatrix ();
  glTranslatef ((float)-0.25, (float)-0.625,  (float)-0.75);
  buildTexturedCube((float)0.05, (float)0.75, (float)0.05);
  glPopMatrix ();

  glPushMatrix ();
  glTranslatef ((float)0.25, (float)-0.25,  (float)-1.25);
  buildTexturedCube((float)0.05, (float)1.5, (float)0.05);
  glPopMatrix ();
   
  glPushMatrix ();
  glTranslatef ((float)-0.25, (float)-0.25, (float)-1.25);
  buildTexturedCube((float)0.05, (float)1.5, (float)0.05);
  glPopMatrix ();
   
  // top
  glPushMatrix ();
  glTranslatef ((float)0.0, (float)0.455, (float)-1.25);
  buildTexturedCube((float)0.5, (float)0.05, (float)0.05);
  glPopMatrix ();  
    
  // middle
  glPushMatrix ();
  glTranslatef ((float)0.0, (float)0.2, (float)-1.25);
  buildTexturedCube((float)0.5, (float)0.05, (float)0.05);
  glPopMatrix ();
   
  glPopMatrix();
   
  glEndList();
}


void buildLightSource(int nbr_y, int nbr_z, float delta_y, float delta_z)
{
  int i,j;
  
  // do for all positions in array  light_position
  
      // set the position of one light
      
	  
      // set the homogene coordinate to 1.0
      // (a default value 0.0 would switch to parallel light)
      
    
   	
}


void drawObjects(void)
{
  // draw table
  glCallList(AL_TABLE_LIST); 
  
  // draw chairs
  glCallList(AL_CHAIR_LIST); 

  glPushMatrix();
  glRotatef(180.0, 0.0, 1.0, 0.0); // rotate 180 degree around the y-axis
  glCallList(AL_CHAIR_LIST);
  glPopMatrix ();
  
  glPushMatrix();
  glRotatef(90.0, 0.0, 1.0, 0.0); // rotate 90 degree around the y-axis
  glTranslatef(0.0, 0.0, 0.0);
  glCallList(AL_CHAIR_LIST);
  glPopMatrix ();
  
  glPushMatrix();
  glRotatef(270.0, 0.0, 1.0, 0.0); // rotate 270 degree around the y-axis
  glTranslatef(0.0, 0.0, 0.0);
  glCallList(AL_CHAIR_LIST);
  glPopMatrix ();
}


void drawTeapot(void)
{
  glPushMatrix();
  glRotatef(90.0, 0.0, 1.0, 0.0); // rotate 90 degree around the y-axis
  glTranslatef((float)0.0, (float)0.15, (float)0.0);
  glutSolidTeapot(0.2);
  glPopMatrix();
}


void generateShadowProjections(GLfloat *projection_matrixX, GLfloat *projection_matrixY, 
			       GLfloat *projection_matrixZ, int light_nbr)
{
  // the current light position is:
  //  light_position[light_nbr][0]
  //  light_position[light_nbr][1]
  //  light_position[light_nbr][2]

  int i;
  // build all three projection matrices
  



}


void drawShadow(GLfloat *projection_matrix, int light_nbr)
{
  // push matrix on current stack and load
  // id matrix on top
  
  
  
  // translate whole scene such that current light position
  // is located at origin
  // (using a 'glTranslatef', then 'glMultMatrixf' and again 'glTranslatef'
  



  // prepare scene for drawing objects relative
  // to CENTER point (translate to CENTER using 'glTranslatef')
  

  // draw table, chairs and teapot
  // (call functions drawObjects and drawTeapot)
  	  

  // pop matrix from current stack
  
}


void displayOnePass(int light_nbr)
{
  GLint viewport[4];
  int i;
  int jitter_sample_nbr;

  // init shadow projection matrices
  GLfloat light_project_matrixX[] = { 1.0,  0.0,  0.0, 1.0,
				      0.0,  1.0,  0.0, 0.0,
				      0.0,  0.0,  1.0, 0.0, 
				      0.0,  0.0,  0.0, 0.0 };
  GLfloat light_project_matrixY[] = { 1.0,  0.0,  0.0, 0.0,
				      0.0,  1.0,  0.0, 1.0,
				      0.0,  0.0,  1.0, 0.0, 
				      0.0,  0.0,  0.0, 0.0 };
  GLfloat light_project_matrixZ[] = { 1.0,  0.0,  0.0, 0.0,
				      0.0,  1.0,  0.0, 0.0,
				      0.0,  0.0,  1.0, 1.0, 
				      0.0,  0.0,  0.0, 0.0 };	

  // set the camera parameters
  glGetIntegerv(GL_VIEWPORT, viewport);
#ifdef AL_ANTI_ALIASING
  jitter_sample_nbr = light_nbr % AL_NBR_JITTER_SAMPLE;
  accPerspective (47.0, (GLdouble) viewport[2]/(GLdouble) viewport[3], 
		  1.0, 40.0, j8[jitter_sample_nbr][0], j8[jitter_sample_nbr][1], 0.0, 0.0, 1.0);
#else
  accPerspective (40.0, (GLdouble) viewport[2]/(GLdouble) viewport[3], 
		  1.0, 50.0, 1.0, 1.0, 0.0, 0.0, 1.0);
#endif

  // set the material for the light
  setMaterial(light_ambient, light_diffuse, light_specular, light_shininess, light_emission);

  // draw light dots
  for(i = 0; i < AL_NBR_OF_LIGHTS; i++)
    {
      glPushMatrix();
      glTranslatef(light_position[i][0], light_position[i][1], light_position[i][2]); 
      glutSolidSphere(0.005, 8, 8);
      glPopMatrix();
    }
	  
  // set light positions
  glLightfv(GL_LIGHT0, GL_POSITION, light_position[light_nbr]);  

  // set material properties for the wall
  setMaterial(wall_ambient, wall_diffuse, wall_specular, wall_shininess, wall_emission);

  // enable texture mapping      
  glEnable(GL_TEXTURE_2D);

  // set wall texture 
  glBindTexture(GL_TEXTURE_2D, wall_texture_id); //select this texture as current

  // draw the first wall
  glPushMatrix();
  glLoadIdentity ();
  glTranslatef ((float)(AL_X_WALL_POS + 0.01),
                (float)(AL_WALL_SIZE/2 + AL_Y_WALL_POS),
                (float)(AL_Z_WALL_POS-AL_WALL_SIZE/2));
  buildTexturedRectangle((float)0.0, (float)(AL_WALL_SIZE+0.1),
                         (float)(AL_WALL_SIZE+0.1), AL_WALL_SUB_DIV);
  glPopMatrix();

  // draw the second wall
  glPushMatrix();
  glLoadIdentity ();
  glTranslatef ((float)(AL_X_WALL_POS - AL_WALL_SIZE/2),
                (float)(AL_WALL_SIZE/2 + AL_Y_WALL_POS),
                (float)(AL_Z_WALL_POS + 0.01));
  buildTexturedRectangle((float)(AL_WALL_SIZE+0.1),
                         (float)(AL_WALL_SIZE+0.1), (float)0.0, AL_WALL_SUB_DIV);
  glPopMatrix();

  // set floor texture
  glBindTexture(GL_TEXTURE_2D, carpet_texture_id); //select this texture as current               

  glPushMatrix();
  glLoadIdentity ();
  glTranslatef ((float)(AL_X_WALL_POS - AL_WALL_SIZE/2),
                (float)(AL_Y_WALL_POS - 0.01),
                (float)(AL_Z_WALL_POS-AL_WALL_SIZE/2));
  buildTexturedRectangle((float)(AL_WALL_SIZE+0.1), (float)0.0,
                         (float)(AL_WALL_SIZE+0.1), AL_WALL_SUB_DIV);
  glPopMatrix();
          
  // set material for the objects
  setMaterial(table_ambient, table_diffuse, table_specular, table_shininess, table_emission);

  // set wood-texture     
  glBindTexture(GL_TEXTURE_2D, wood_texture_id); //select this texture as current

  // translate to object center
  glPushMatrix();
  glLoadIdentity ();
  glTranslatef (AL_CENTER_X, AL_CENTER_Y, AL_CENTER_Z);
          
  // draw table chairs
  drawObjects();

  // set material properties for teapot
  glDisable(GL_TEXTURE_2D);       
  setMaterial(teapot_ambient, teapot_diffuse, teapot_specular, teapot_shininess, teapot_emission);

  // draw teapot
  drawTeapot();

  // generate shadow projection matrices
  generateShadowProjections(light_project_matrixX, light_project_matrixY, light_project_matrixZ, light_nbr);

  // set material parameters for shadow
  setMaterial(shadow_ambient, shadow_diffuse, shadow_specular, shadow_shininess, shadow_emission);        

  // draw shadows
  glMatrixMode(GL_MODELVIEW);
  drawShadow(light_project_matrixX, light_nbr);
  drawShadow(light_project_matrixY, light_nbr);
  drawShadow(light_project_matrixZ, light_nbr);

  // back of the object center...
  glPopMatrix();
}


void init(void)
{
  // define a light and set ligthing properties
  setLighting();

  // define default light position
  light_position[0][0] = (float)AL_LIGHT_POS_X;
  light_position[0][1] = (float)AL_LIGHT_POS_Y;
  light_position[0][2] = (float)AL_LIGHT_POS_Z;
  light_position[0][3] = 1.0;

  // construct a spatial light source with n single lights
  buildLightSource(AL_NBR_Y_LIGHTS, AL_NBR_Z_LIGHTS, (float)AL_DELTA_Y_LIGHTS, (float)AL_DELTA_Z_LIGHTS);                                                        
  // set & enable depth buffering
  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
  
  // set shading model (gouraud)
  glShadeModel(GL_SMOOTH);

  // read textures
  printf("\nReading texture images ...");
  read_texmap("Textures/wall_texture.PPM", &wall_texture_width, &wall_texture_height, wall_texture);
  read_texmap("Textures/carpet_texture.PPM", &carpet_texture_width, &carpet_texture_height, carpet_texture);
  read_texmap("Textures/wood2_texture.PPM", &wood_texture_width, &wood_texture_height, wood_texture);
  printf(" done.\n\n");
   
  // load texture into the texture memory and reference it by an id
  glBindTexture(GL_TEXTURE_2D, wall_texture_id);  
  glTexImage2D(GL_TEXTURE_2D, 0, 3, wall_texture_width, wall_texture_height, 0, GL_RGB, GL_UNSIGNED_BYTE, 
               &wall_texture);  
                    
  // set texture parameters
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  
  // load texture into the texture memory and reference it by an id
  glBindTexture(GL_TEXTURE_2D, carpet_texture_id); //select this texture as current               
 
  glTexImage2D(GL_TEXTURE_2D, 0, 3, carpet_texture_width, carpet_texture_height, 0, GL_RGB, GL_UNSIGNED_BYTE, 
               &carpet_texture); 
               
  // set texture parameters
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  
  // load texture into the texture memory and reference it by an id
  glBindTexture(GL_TEXTURE_2D, wood_texture_id); //select this texture as current                         
                
  glTexImage2D(GL_TEXTURE_2D, 0, 3, wood_texture_width, wood_texture_height, 0, GL_RGB, GL_UNSIGNED_BYTE, 
               &wood_texture);            
 
  // set texture parameters
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  

  // set clear values
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glClearAccum(0.0, 0.0, 0.0, 0.0);

  // build display lists
  buildTable();
  buildChair();
}


//callback functions :
void callback_display(void)
{
  int light_nbr = 0;

  // clear the accumulation buffer
  
   
  // for each light render the scene (0 <= light_nbr < AL_NBR_OF_LIGHTS)
  
    // clear the color buffer and the depth buffer
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

    // render the scene for one light
    displayOnePass(light_nbr);
	
    // accumulate the rendered picture
	// using 'glAccum' and parameter GL_ACCUM and ...)
    
  
  
  // write the accumulation buffer into the actual color buffer
  // using 'glAccum'
  
   
  // display the actual color buffer
  glFlush();
}


void callback_reshape(int w, int h)
{
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glClearAccum(0.0, 0.0, 0.0, 0.0);
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
}


//main :
int main(int argc, char** argv)
{ 
  glutInit(&argc, argv);
  glutInitDisplayMode (GLUT_SINGLE|GLUT_RGB|GLUT_ACCUM|GLUT_DEPTH);
  glutInitWindowSize (600, 400);
  glutCreateWindow (argv[0]);
  init();
  glutReshapeFunc(callback_reshape);
  glutDisplayFunc(callback_display);
  glutMainLoop();
  return 0;
}
